<?php

namespace Ardent\PipelineBundle\Pipeline\OptionsProcessor;

class OptionsStageBuilder
{
    /**
     * @var OptionsStageInterface[]
     */
    private array $stages = [];

    public function __construct(
        private readonly OptionsStageBag $bag,
        private readonly ?Options        $options = null,
    )
    {
    }

    /**
     * @param string|OptionsStageInterface $stageClass Can either be the classFQN of an OptionsStageInterface or one directly
     */
    public function add(string|OptionsStageInterface $stageClass): self
    {
        if (is_string($stageClass)) {
            $this->stages[] = $this->bag->get($stageClass);
        } else {
            $this->stages[] = $stageClass;
        }

        return $this;
    }

    public function each(ItemsOptionsPipeline $pipeline): self
    {
        $this->stages[] = $this->bag->get(SubItemStage::class)->setSubPipeline($pipeline);

        return $this;
    }

    public function build(): ItemsOptionsPipeline
    {
        return new ItemsOptionsPipeline(new OptionsProcessor($this->options), ...$this->stages);
    }
}